; Tastaturconverter mit Mikrocontroller AT 89 C 2051
; Die Signale einer PC-AT Tastatur werden in ASCII-
; Zeichen fuer die serielle Schnittstelle gewandelt
; Pin 16 Daten P1.5   Pin 17 Takt P1.4
; Quarzfrequenz 11,0592 MHz
; P1.0 bis P1.3  dient der Wahl der Baudrate
; R4 speichert Zustand der Shift Taste
; mit belegten Funktionstasten F1 ... F12
; im Beispiel sind nur F1 bis F6 belegt
; Vereinbarungen
	BSEG
TCLK:	EQU 94H			;Tastaturtakt
TDAT:   EQU 95H			;Tastaurdaten
TR1:    EQU 8EH			;Startflag fuer Timer 1
TI:     EQU 99H			;Sendeflag
RI:     EQU 98H			;Empfangsflag
PA:	EQU 0D0H		;Parityflag
ACC.7:  EQU 0E7H		;Akku Bit 7
;
        DSEG
SCON: EQU 98H			;Controllregister serielle Betriebsart
TMOD: EQU 89H			;Controllregister Timer 1
PCON: EQU 87H                   ;SMOD
TH1:  EQU 8DH                   ;Werteregister Timer 1
SBUF: EQU 99H			;Sendepuffer
P1:   EQU 90H			;Port 1
B:    EQU 0F0H			;Hilfsregister B
;
        CSEG
	ORG 0000H
ANF:    MOV P1,#0FFH		;alles high Anfangsbedingung
	MOV R4,#00H             ;Anfangswert Shift
	MOV SCON,#50H		;Serielle Schnittstelle Modus 1
        MOV TMOD,#20H		;Timer 1 Mode 2  Timer 0 keine Funktion
        MOV PCON,#00H           ;Schnittstellenmodus bidi setzen
	ACALL BDRATE		;Baudrate festlegen
        MOV TH1,A               ;Nachladewert fr Baudrate bei 11,0592 MHz
        SETB TR1                ;Timer 1 starten
        SETB TI			;Sendeflag freigeben
	MOV R2,#0FFH		;lange Zeitschleife
AN1:	ACALL WAIT              ;warten
	DJNZ R2,AN1		;Resetzeit fuer Tastatur
	MOV A,#0FFH		;Reset Tastatur
	ACALL TASE		;zur Tastatur senden
	ACALL WAIT              ;ACK ausblenden
	MOV A,#0F0H		;Scancode festlegen
	ACALL TASE		;zur Tastatur senden
	ACALL WAIT		;ACK ausblenden
	MOV A,#03H              ;Scancode 3
	ACALL TASE		;zur Tastatur senden
	ACALL WAIT              ;ACK ausblenden
	MOV A,#0EDH		;Leuchtdiodenbefehl
	ACALL TASE		;zur Tastatur senden
	ACALL WAIT              ;ACK ausblenden
	MOV A,#00H		;alle LED aus
	ACALL TASE              ;zur Tastatur senden
        ACALL WAIT              ;ACK ausblenden
HP1:    ACALL TAEM		;von Tastatur lesen
        CJNE A,#0F0H,HP2	;auf Break Code vergleichen
	LJMP HP1                ;Break Code ausblenden
HP2:    CJNE A,#07H,HP3		;Funktionstaste F1 ?
	LJMP FP1		;FUNKTION NEW
HP3:	CJNE A,#0FH,HP4		;Funktionstaste F2 ?
	LJMP FP2		;FUNKTION LIST
HP4:    CJNE A,#17H,HP5         ;Funktionstaste F3 ?
	LJMP FP3                ;FUNKTION PRINT
HP5:   	CJNE A,#1FH,HP6         ;Funktionstaste F4 ?
	LJMP FP4		;FUNKTION CLS
HP6:	CJNE A,#27H,HP7		;Funktionstaste F5 ?
	LJMP FP5		;FUNKTION RENUMBER
HP7:	CJNE A,#2FH,HP8         ;Funktionstaste F6 ?
        LJMP FP6		;FUNKTION SAVE
HP8:    ANL A,#7FH		;nur bis 7FH
	MOV B,A 		;Zeichen sichern
	ACALL SHIFT		;Shift Tasten beruecksichtigen
	ACALL UMCO		;umcodieren
	CJNE A,#00H,HP9		;auf Code 00 kontrollieren
	LJMP HP1		;Code 00 ignorieren
HP9:	ACALL SEND		;ueber V.24 uebertragen
	LJMP HP1
;
; eigentliche Senderoutine als Unterprogramm
; das Sendezeichen steht in A
;
SEND:   JNB TI,SEND             ;Sendeflag pruefen bis leer
        CLR TI			;Sendepuffer loeschen
        MOV SBUF,A              ;Zeichen zum Sendepuffer
        RET

;
; eigentliche Empfangsroutine als Unterprogramm
; das empfangene Zeichen steht in A
;
EMPF:   JNB RI,EMPF		;Empfangsflag pruefen
        MOV A,SBUF              ;empfangenes Zeichen nach A
        CLR RI                  ;Empfangsflag loeschen
        RET
;
; Warteschleife
;
WAIT:	MOV R7,#0FFH 		;Zeitschleifenwert
WAIT1:	DJNZ R7,WAIT1		;warten
        MOV R7,#0FFH		;Zeitschleifenwert
WAIT2:  DJNZ R7,WAIT2		;warten
	MOV R7,#0FFH
WAIT3:  DJNZ R7,WAIT3
	RET
;
; Baudrate entsprechend Schiebeschaltern festlegen
;
BDRATE: MOV A,P1		;Port 1 einlesen
        ANL A,#0FH	        ;nur untere Bits auswerten
        MOV DPTR,#BRTAB  	;Anfang Baudratentabelle
	MOVC A,@A+DPTR		;Konstante Baudrate holen
	RET
;
; Senden von Daten zur Tastatur
;
TASE:	MOV C,PA		;Parity nach Carry
	CPL C			;Paritaet invertieren
	MOV R0,#00H		;Bitzaehler
        CLR TDAT		;Tastaturdatenleitung auf Low
TASE1:  JB TCLK,TASE1		;warten auf Wechsel nach Low
	RRC A			;rotieren
	MOV TDAT,C		;Bit ausgeben
TASE2:  JNB TCLK,TASE2		;Warten auf Taktwechsel nach High
	INC R0			;Bitzaehler erhoehen
	CJNE R0,#0AH,TASE1	;bis zum letzten Bit
	SETB TDAT		;Tastaturdaten setzen
TASE3:  JB TCLK,TASE3		;Taktwechsel nach Low fr Stoppbit
TASE4:	JNB TCLK,TASE4		;Taktwechsel nach High abwarten
	ACALL WAIT              ;ACK von Tastatur abwarten
	RET
;
; Empfangen von Daten von der Tastatur
;
TAEM:	SETB TDAT		;Takt auf High
TAEM1:	JB TCLK,TAEM1           ;Warten auf Takt = Low
TAEM2:  JNB TCLK,TAEM2		;Warten auf Ende Startbit
	MOV R0,#00H		;Bitzaehler
TAEM3:  JB TCLK,TAEM3		;bis richtiger Taktzustand
	MOV C,TDAT		;Datenbit von Tast nach Carry
TAEM4:  JNB TCLK,TAEM4		;Warten auf Takt
	RRC A			;Bits rotieren
	INC R0			;Bitzaehler erhoehen
	CJNE R0,#08H,TAEM3	;bis Byte fertig
TAEM5:  JB TCLK,TAEM5		;bis richtiger Taktzustand
	MOV C,TDAT		;Parity einlesen
TAEM6:  JNB TCLK,TAEM6		;
	ACALL WAIT		;
	RET
;
; Shift Taste auswerten und zur Umcodierung merken
;
SHIFT:  CJNE A,#12H,SH1		;Code 12 fuer Shift
SH3:	MOV A,R4		;Zustand Shift nach A
	CPL ACC.7		;Zustand Shift aendern
	MOV R4,A		;Zustand Shift speichern
	RET
SH1:    CJNE A,#59H,SH2		;Code 59 fuer Shift
        JMP SH3
SH2:    RET
;
; Umcodierung des Scan-Codes 3 in ASCII Zeichen
;
UMCO:	MOV A,#00H		;Offset = 0
	MOV DPTR,#CODT		;Anfang Codetabelle
	ADD A,B			;Codeoffset berechnen
        ADD A,R4		;Shift beruecksichtigen
	MOVC A,@A+DPTR 		;Element holen
	RET
;
; Codetabelle
;
; Zeichen 00H ... 0FH   ohne Shift
;
CODT:   DEFB 00H, 00H, 00H, 00H		;00 00 00 00
	DEFB 00H, 00H, 00H, 00H         ;00 00 00 00
        DEFB 1BH, 00H, 00H, 00H         ;ESC 00 00 00
	DEFB 00H, 00H, 7FH, 00H		;00 00 00 00
; Zeichen 10H ... 1FH
	DEFB 00H, 00H, 00H, 3EH		;00 00 00 <
	DEFB 00H, 71H, 31H, 00H         ;00 q 1 00
	DEFB 00H, 00H, 79H, 73H         ;00 00 y s
	DEFB 61H, 77H, 32H, 00H         ;a w 2 00
; Zeichen 20H ... 2FH
	DEFB 00H, 63H, 78H, 64H		;00 c x d
	DEFB 65H, 34H, 33H, 00H		;e 4 3 00
	DEFB 00H, 20H, 76H, 66H	        ;00 Space v f
	DEFB 74H, 72H, 35H, 00H		;t r 5 00
; Zeichen 30H ... 3FH
	DEFB 00H, 6EH, 62H, 68H		;00 n b h
	DEFB 67H, 7AH, 36H, 00H         ;g z 6 00
	DEFB 00H, 00H, 6DH, 6AH         ;00 00 m j
	DEFB 75H, 37H, 38H, 00H         ;u 7 8 00
; Zeichen 40H ... 4FH
	DEFB 00H, 2CH, 6BH, 69H		;00 , k i
	DEFB 4FH, 30H, 39H, 00H		;O 0 9 00
	DEFB 00H, 2EH, 2DH, 6CH		;00 . - l
	DEFB 7CH, 70H, 7EH, 00H		; p  00
; Zeichen 50H ... 5FH
        DEFB 00H, 00H, 7BH, 23H         ;00 00  #
	DEFB 7DH, 60H, 00H, 00H         ; ' 00 00
        DEFB 00H, 00H, 0DH, 2BH         ;00 00 ENTER +
	DEFB 00H, 00H, 00H, 00H         ;00 00 00 00
; Zeichen 60H ... 6FH
        DEFB 0AH, 08H, 00H, 0BH         ;LF BS 00 VT
        DEFB 00H, 00H, 00H, 00H         ;00 00 00 00
	DEFB 00H, 31H, 09H, 34H		;00 1 TAB 4
	DEFB 37H, 00H, 00H, 00H		;00 00 00 00
; Zeichen 70H ... 7FH
	DEFB 30H, 00H, 32H, 35H		;0 00 2 5
	DEFB 36H, 38H, 00H, 00H		;6 8 00 00
	DEFB 00H, 0DH, 33H, 00H		;00 ENTER 3 00
	DEFB 00H, 39H, 00H, 00H		;00 9 00 00
;
; Zeichen 00H ... 0FH  mit Shift
        DEFB 00H, 00H, 00H, 00H		;00 00 00 00
	DEFB 00H, 00H, 00H, 00H         ;00 00 00 00
        DEFB 1BH, 00H, 00H, 00H         ;ESC 00 00 00
	DEFB 00H, 00H, 7FH, 00H		;00 00 7FH 00
; Zeichen 10H ... 1FH mit Shift
	DEFB 00H, 00H, 00H, 3CH		;00 00 00 >
	DEFB 00H, 51H, 21H, 00H         ;00 Q ! 00
	DEFB 00H, 00H, 59H, 53H         ;00 00 Y S
	DEFB 41H, 57H, 22H, 00H         ;A W " 00
; Zeichen 20H ... 2FH  mit Shift
	DEFB 00H, 43H, 58H, 44H		;00 C X D
	DEFB 45H, 24H, 23H, 00H		;E $ $ 00
	DEFB 00H, 20H, 56H, 46H	        ;00 Space V F
	DEFB 54H, 52H, 25H, 00H		;T R % 00
; Zeichen 30H ... 3FH  mit Shift
	DEFB 00H, 4EH, 42H, 48H		;00 N B H
	DEFB 47H, 5AH, 26H, 00H         ;G Z & 00
	DEFB 00H, 00H, 4DH, 4AH         ;00 00 M J
	DEFB 55H, 27H, 28H, 00H         ;U / ( 00
; Zeichen 40H ... 4FH  mit Shift
	DEFB 00H, 3BH, 4BH, 49H		;00 ; K I
	DEFB 4FH, 3DH, 29H, 00H		;O = ) 00
	DEFB 00H, 3AH, 5FH, 4CH		;00 : _ L
	DEFB 5CH, 50H, 3FH, 00H		; P ? 00
; Zeichen 50H ... 5FH  mit Shift
        DEFB 00H, 00H, 5BH, 23H         ;00 00  #
	DEFB 5DH, 27H, 00H, 00H         ; ' 00 00
        DEFB 00H, 00H, 0DH, 2AH         ;00 00 ENTER *
	DEFB 00H, 00H, 00H, 00H         ;00 00 00 00
; Zeichen 60H ... 6FH  mit Shift
        DEFB 0AH, 08H, 00H, 0BH         ;LF BS 00 VT
        DEFB 00H, 00H, 00H, 00H         ;00 00 00 00
	DEFB 00H, 31H, 09H, 34H		;00 1 TAB 4
	DEFB 37H, 00H, 00H, 00H		;00 00 00 00
; Zeichen 70H ... 7FH  mit Shift
	DEFB 30H, 00H, 32H, 35H		;0 00 2 5
	DEFB 36H, 38H, 00H, 00H		;6 8 00 00
	DEFB 00H, 0DH, 33H, 00H		;00 ENTER 3 00
	DEFB 00H, 39H, 00H, 00H		;00 9 00 00
;
; Baudratentabelle
;
BRTAB:  DEFB 0FDH	   		;9600 Baud  0
	DEFB 0FAH			;4800 Baud  1
	DEFB 0F4H			;2400 Baud  2
	DEFB 0E8H			;1200 Baud  3
	DEFB 0D0H			;600 Baud   4
	DEFB 0A0H			;300 Baud   5
	DEFB 040H			;150 Baud   6
	DEFB 0FDH			;9600 Baud  7
        DEFB 0FDH			;9600 Baud  8
	DEFB 0FDH			;9600 Baud  9
	DEFB 0FDH			;9600 Baud  A
	DEFB 0FDH                       ;9600 Baud  B
	DEFB 0FDH			;9600 Baud  C
	DEFB 0FDH			;9600 Baud  D
	DEFB 0FDH			;9600 Baud  E
	DEFB 0FDH			;9600 Baud  F
;
; Funktionstastenunterprogramme
;
FP1:    MOV R5,#04H			;4 Zeichen
	MOV DPTR,#ZK1  			;Anfang Zeichenkette
FFP1:	MOV A,#00H			;Offset = 0
	MOVC A,@A+DPTR			;Zeichen holen
	LCALL SEND			;Zeichen senden
	INC DPTR			;naechstes Zeichen
	DJNZ R5,FFP1                    ;bis zum Ende
	LJMP HP1
ZK1:    DEFB 'N'
	DEFB 'E'
	DEFB 'W'
	DEFB 0DH
FP2:    MOV R5,#05H			;5 Zeichen
	MOV DPTR,#ZK2			;Anfang Zeichenkette
FFP2:   MOV A,#00H			;Offset = 0
	MOVC A,@A+DPTR			;Zeichen holen
	LCALL SEND			;Zeichen senden
	INC DPTR			;naechstes Zeichen
	DJNZ R5,FFP2			;bis zum Ende
	LJMP HP1
ZK2:    DEFB 'L'
	DEFB 'I'
	DEFB 'S'
	DEFB 'T'
	DEFB 0DH
FP3:    MOV R5,#06H			;6 Zeichen
        MOV DPTR,#ZK3			;Anfang Zeichenkette
FFP3:   MOV A,#00H			;Offset = 0
	MOVC A,@A+DPTR			;Zeichen holen
	LCALL SEND			;Zeichen senden
        INC DPTR			;naechstes Zeichen
	DJNZ R5,FFP3			;bis zum Ende
	LJMP HP1                        ;
ZK3:    DEFB 'P'
	DEFB 'R'
	DEFB 'I'
	DEFB 'N'
	DEFB 'T'
	DEFB 0DH
FP4:    MOV R5,#04H			;4 Zeichen
	MOV DPTR,#ZK4			;Anfang Zeichenkette
FFP4:	MOV A,#00H			;Offset = 0
	MOVC A,@A+DPTR			;Zeichen holen
	LCALL SEND			;Zeichen senden
	INC DPTR			;naechstes Zeichen
	DJNZ R5,FFP4			;bis zum Ende
	LJMP HP1			;
ZK4:	DEFB 'C'
	DEFB 'L'
	DEFB 'S'
	DEFB 0DH
FP5:    MOV R5,#09H			;9 Zeichen
	MOV DPTR,#ZK5                   ;Anfang Zeichenkette
FFP5:   MOV A,#00H			;Offset = 0
	MOVC A,@A+DPTR			;Zeichen holen
	LCALL SEND			;Zeichen senden
	INC DPTR			;naechstes Zeichen
	DJNZ R5,FFP5			;bis zum Ende
	LJMP HP1			;
ZK5:    DEFB 'R'
	DEFB 'E'
	DEFB 'N'
	DEFB 'U'
	DEFB 'M'
	DEFB 'B'
	DEFB 'E'
	DEFB 'R'
	DEFB 0DH
FP6:    MOV R5,#05H			;5 Zeichen
        MOV DPTR,#ZK6			;Anfang Zeichenkette
FFP6:   MOV A,#00H			;Offset = 0
	MOVC A,@A+DPTR			;Zeichen holen
	LCALL SEND			;Zeichen senden
	INC DPTR			;naechstes Zeichen
	DJNZ R5,FFP6			;bis zum Ende
	LJMP HP1			;
ZK6:    DEFB 'S'
	DEFB 'A'
	DEFB 'V'
	DEFB 'E'
	DEFB 0DH
;
	DEFB 00H
	DEFB 00H
;
;
	DB 0DH
	DB 0AH
	DB 0DH
        DB 0AH
